% // Sensoray 2600 matlab example
% // load the 2600 library/dll
% // make sure correct dll is in Matlab path (32-bit for 32-bit Matlab,
% // 64-bit for 64-bit Matlab)

loadlibrary('s2600.dll', 'app2600.h', 'addheader','s26app.h', ...
	    'alias','S2600Lib');

% // display the current DLL version
[version_str] = calllib('S2600Lib', 'S26_DriverVersion');

% open the dll 
rc = calllib('S2600Lib', 'S26_DLLOpen', 1);

% //set up io module status info pointer.
% //Due to how parameters are passed in Matlab, do not use null or zero
% //for iomstatus as a value even if unused.

% // Iom status info. declare allocated pointer space for DLL
iom_status_ptr    = libpointer('uint8Ptr',zeros(16,1,'uint8'));

disp("S2600 libraries loaded. Version: " + version_str);

% // Open the MM at the default IP address
% // In this example, only one MM is present in the system
% // so zero-indiced mmhandle is 0.

mmhandle = 0;
ipaddr = '10.10.10.1';

rc = calllib('S2600Lib','S26_BoardOpen', mmhandle, '0', ipaddr);
% // retry connection if call failed
while rc ~= 0
  rc = calllib('S2600Lib','S26_BoardOpen', mmhandle, '0', ipaddr);
  pause(0.25);
  disp("Attempting to establish a connection with the MM.");
end

disp("MM was successfully opened.");

% // Reset the I/O system.
rc = calllib('S2600Lib','S26_ResetNetwork', mmhandle);

while rc == 0
  rc = calllib('S2600Lib','S26_ResetNetwork', mmhandle);
  pause(0.25);
end

disp("MM was successfully reset.");

% // Disable automatic reset (watchdog) if desired
sched_ptr = calllib('S2600Lib','S26_SchedOpen',0,1);
rc = calllib('S2600Lib','S26_Sched2601_SetWatchdog',sched_ptr,0); % Disable watchdog
% // call schedexecute to complete transaction(s)
[rc, out_void_ptr, out_ptr] = calllib('S2600Lib','S26_SchedExecute',sched_ptr, 1000, iom_status_ptr);

if rc ~= 0
  disp("Watchdog disable failed.");
 else
   disp("Watchdog was successfully disabled.");   
end

% //Detect and register all i/o modules connected to the 2601 main module.


msec_gateway    = 100;	% //timeout/retry for gateway (ms)
msec_comport    = 100;	% //timeout/retry for comport transactions (ms)
com_retries     = 20;	% // number of comport retries.
gateway_retries = 20;	% // number of gateway retries.

nboards_ptr     = libpointer('uint16Ptr',0);                    % Number of detected iom's.
iomtype_ptr     = libpointer('uint16Ptr',zeros(16,1,'uint16')); % Detected iom types.


[rcs, num_ioms_ptr, ioms_out_ptr, status_out_ptr] = calllib('S2600Lib','S26_RegisterAllIoms', mmhandle, msec_gateway, nboards_ptr, iomtype_ptr, iom_status_ptr, gateway_retries);

% // Set encoder mode settings
% // In this example, channels are set to quadrature encoding
clockMode = 3; %// 3 = quadrature x4, clock on all A and B edges.

  % // Index pin polarity
indexPolarity = 0; % Index pin polarity: 1 = active low, 0 = active high.

for IOM = 0 : (num_ioms_ptr-1)
    sched_ptr = calllib('S2600Lib','S26_SchedOpen',0,1);
    pause(1);
    [rc, void_ptr] = calllib('S2600Lib','S26_Sched2620_SetModeEncoder',sched_ptr,IOM,0,indexPolarity,0,clockMode);
    pause(1);
    [rc, out_void_ptr, out_ptr] = calllib('S2600Lib','S26_SchedExecute',sched_ptr, 1000, iom_status_ptr);
    disp(rc);
    pause(1);
end   

% // Read Encoders
% // Latch
CTC_TRIG_PRELOAD = 1;
CTC_TRIG_LATCH = 2;

counts_ptr      = libpointer('uint32Ptr',0);  % Counts.
timestamp_ptr   = libpointer('uint16Ptr',0);  % Timestamps.

sched_ptr = calllib('S2600Lib','S26_SchedOpen',0,1);
[rc, void_ptr] = calllib('S2600Lib','S26_Sched2620_SetControlReg',sched_ptr,1,0,CTC_TRIG_LATCH);
pause(1);
[rc, out_void_ptr, out_ptr] = calllib('S2600Lib','S26_SchedExecute',sched_ptr, 1000, iom_status_ptr);
pause(1);
disp(rc);

quad_inputs = 4; % // Available channels on 2620 modules
% // allocate the count list in advance
count_list = zeros(num_ioms_ptr,quad_inputs);
disp("CountList before measurement:");
disp(count_list);

% // Fetch counter values of attached 2620s
for iom = 0 : (num_ioms_ptr-1)
    for chan = 0:(quad_inputs-1) 
        sched_ptr = calllib('S2600Lib','S26_SchedOpen',0,1);
        [rc, void_ptr, counts, TimeStamp] = calllib('S2600Lib','S26_Sched2620_GetCounts',sched_ptr,iom,chan,counts_ptr,timestamp_ptr);
        pause(1);
	[rc, out_void_ptr, out_ptr] = calllib('S2600Lib','S26_SchedExecute',sched_ptr, 1000, iom_status_ptr);
        count_list(iom+1,chan+1) = counts;
        disp(counts);
    end
end


disp(" ");
disp("Counts:");
disp(" ");
disp(count_list);

% // Clean up and close the DLL
% // Note: Do not use S26_BoardClose, it will be called internally
% // in S26_DLLClose
% //Warning. S26_DLLClose will hang if S26_SchedExecute is not called after 
% //initializing a transaction (with SchedOpen, GetCounts, etc...)
% //Make sure all SchedOpen calls are completed.

calllib('S2600Lib','S26_DLLClose')
unloadlibrary S2600Lib
