VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClsStdDev"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'*************************************************************************
' Module    : MyStdDev.cls
' Function  : Defines ClsMyStdDev, used for computing standard deviations.
' Author    : Jim Lamberson
' Copyright : (C) 2002 Sensoray
'*************************************************************************

Option Explicit

' Configuration properties.
Private MySampleSize    As Long

' Dynamic properties.
Private MyAbsMax        As Double
Private MyAbsMin        As Double

' Runtime storage.
Private CurMax          As Double
Private CurMin          As Double
Private SampleSum       As Double
Private MySampleNum     As Long
Private SampleData()    As Double

Event NewResult(MinVal As Double, MaxVal As Double, Mean As Double, StdDev As Double)

Private Sub Class_Initialize()
    SampleSize = 100        ' Default sample size.
End Sub

Public Sub Restart()
    MyAbsMax = -1E+24
    MyAbsMin = 1E+24
    BeginNextStdDev
    RaiseEvent NewResult(0, 0, 0, 0)
End Sub

Public Sub BeginNextStdDev()
    CurMax = -1E+24
    CurMin = 1E+24
    MySampleNum = 1
    SampleSum = 0
End Sub

Public Sub AddSample(ByRef Value As Double)
    Dim i       As Long
    Dim DevSum  As Double
    Dim Mean    As Double
    ' Update current min/max.
    If (Value > CurMax) Then CurMax = Value
    If (Value < CurMin) Then CurMin = Value
    ' Update absolute min/max.
    If (Value > MyAbsMax) Then MyAbsMax = Value
    If (Value < MyAbsMin) Then MyAbsMin = Value
    ' Collect the new sample.
    SampleData(MySampleNum) = Value
    SampleSum = SampleSum + Value
    MySampleNum = MySampleNum + 1
    ' Compute Mean and StdDev if we have enough samples.
    If (MySampleNum > MySampleSize) Then
        Mean = SampleSum / MySampleSize
        For i = 1 To MySampleSize
            DevSum = DevSum + (Mean - SampleData(i)) ^ 2
        Next i
        ' Notify client that new values are available.
        RaiseEvent NewResult(CurMin, CurMax, Mean, 0.6745 * Sqr(DevSum) / Sqr(MySampleSize - 1))
        ' Start next acquisition.
        BeginNextStdDev
    End If
End Sub

Public Property Get SampleSize() As Long
    SampleSize = MySampleSize
End Property

Public Property Let SampleSize(ByVal Value As Long)
    If (Value < 2) Then Value = 2
    MySampleSize = Value
    ReDim SampleData(1 To MySampleSize) As Double
    Restart
End Property

Public Property Get SampleCount() As Long
    SampleCount = MySampleNum - 1
End Property

Public Property Get MinValue() As Double
    MinValue = MyAbsMin
End Property

Public Property Get MaxValue() As Double
    MaxValue = MyAbsMax
End Property

